#------------------------------------------------------------------------------------------------
# Permission to use, copy, modify, and distribute this software for 
# any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#------------------------------------------------------------------------------------------------
#    Name:	Intersection of Groups
#      By:	sdmitch
#   Usage:	Intersect two or more groups and collect the intersection edges in a group.
#		 Note:	
#			
#    Date:	Oct 2011
#------------------------------------------------------------------------------------------------

require 'sketchup'

if( not file_loaded?(File.basename(__FILE__)) )
    UI.menu("Plugins").add_item("Intersection of Groups") { Sketchup.active_model.select_tool Intersection_of_Groups.new }
	file_loaded(File.basename(__FILE__))
end

	
class Intersection_of_Groups

	def initialize
		@ip = Sketchup::InputPoint.new
		@ip1 = Sketchup::InputPoint.new
		@Mod = Sketchup.active_model
		@Ent = @Mod.active_entities
		@Sel = @Mod.selection
		self.reset
	end

	def reset
		@grps=[]
		@best = nil
		@ip1.clear
		@Sel.clear
		@drawn = false
		Sketchup::set_status_text("", SB_VCB_LABEL)
		Sketchup::set_status_text("", SB_VCB_VALUE)
	end

	def set_current_point(x, y, view)
		if( !@ip.pick(view, x, y, @ip1) )
			return false
		end
		@ip1.copy! @ip
		view.tooltip = @ip.tooltip
		Sketchup::set_status_text("Select base group", SB_PROMPT) if @grps.length==0
		Sketchup::set_status_text("Select group to intersect", SB_PROMPT) if @grps.length==1
		Sketchup::set_status_text("Select group to intersect or press Enter/Return to execute", SB_PROMPT) if @grps.length>1
	end

	def onMouseMove(flags, x, y, view)
		self.set_current_point(x, y, view)
		ph = view.pick_helper;ph.do_pick(x, y)
		@best = ph.best_picked if ph.best_picked != @best
		view.invalidate
	end

	def onLButtonDown(flags, x, y, view)
		if @best.is_a?(Sketchup::Group)
			@grps.push @best
			@Sel.add @best
		end
	end
	
	def onReturn(view)
		ntersect=@Ent.add_group
		ntersect.name="Intersection";@Sel.clear;
		@Mod.start_operation('Intersection of Groups')
		for i in 1...@grps.length
			@grps[0].entities.intersect_with(true,@grps[0].transformation,ntersect,ntersect.transformation,true,@grps[i])
		end
		@Sel.add ntersect
		@Mod.commit_operation
		Sketchup.send_action "selectSelectionTool:"
	end
	
	def onCancel(flag, view)
		view.invalidate if @drawn
		Sketchup.send_action "selectSelectionTool:"
	end
	
	def onRButtonDown(flags,x,y,view)
		Sketchup.send_action "selectSelectionTool:" if @state == 0
		self.reset
	end

	def draw(view)
		@drawn = false
		if( @ip.valid? && @ip.display? )
			@ip.draw(view)
			@drawn = true
		end
		view.invalidate if @drawn
	end

end # of class Intersection_of_Groups

